@@ -1,4 +1,3 @@
-{-# LANGUAGE TupleSections #-}
 -- incomplete patterns in 'from{Asc,Desc}List'
 {-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
 
@@ -7,6 +6,9 @@ module Mini.Data.Set (
   -- * Type
   Set,
 
+  -- * Primitive Recursion
+  set,
+
   -- * Construction
   empty,
   singleton,
@@ -148,7 +150,7 @@ instance (Show a) => Show (Set a) where
   show = show . toAscList
 
 instance Foldable Set where
-  foldr f b = set b go go go
+  foldr f b = set' b go go go
    where
     go l a _ _ recr = foldr f (f a recr) l
 
@@ -162,23 +164,42 @@ instance (Ord a) => Monoid (Set a) where
  - Primitive recursion
  -}
 
--- | Primitive recursion on sets
+-- | Primitive recursion on sets (internally structured as trees)
 set
   :: b
-  -- ^ Empty node
+  -- ^ Value yielded in case of empty node
   -> (Set a -> a -> Set a -> b -> b -> b)
-  -- ^ Left-heavy node
-  -> (Set a -> a -> Set a -> b -> b -> b)
-  -- ^ Balanced node
-  -> (Set a -> a -> Set a -> b -> b -> b)
-  -- ^ Right-heavy node
+  -- ^ Function applied in case of non-empty node:
+  -- left child, element, right child, left recursion, right recursion
+  -- (elements are lesser to the left, greater to the right)
   -> Set a
-  -- ^ Set
+  -- ^ Object of the case analysis
   -> b
-set e _ _ _ E = e
-set e f g h (L l a r) = f l a r (set e f g h l) (set e f g h r)
-set e f g h (B l a r) = g l a r (set e f g h l) (set e f g h r)
-set e f g h (R l a r) = h l a r (set e f g h l) (set e f g h r)
+set e f = set' e f f f
+
+-- Primitive recursion on sets
+set'
+  :: b
+  -- ^ Value yielded in case of empty node
+  -> (Set a -> a -> Set a -> b -> b -> b)
+  -- ^ Function applied in case of left-heavy node:
+  -- left child, element, right child, left recursion, right recursion
+  -- (elements are lesser to the left, greater to the right)
+  -> (Set a -> a -> Set a -> b -> b -> b)
+  -- ^ Function applied in case of balanced node:
+  -- left child, element, right child, left recursion, right recursion
+  -- (elements are lesser to the left, greater to the right)
+  -> (Set a -> a -> Set a -> b -> b -> b)
+  -- ^ Function applied in case of right-heavy node:
+  -- left child, element, right child, left recursion, right recursion
+  -- (elements are lesser to the left, greater to the right)
+  -> Set a
+  -- ^ Object of the case analysis
+  -> b
+set' e _ _ _ E = e
+set' e f g h (L l a r) = f l a r (set' e f g h l) (set' e f g h r)
+set' e f g h (B l a r) = g l a r (set' e f g h l) (set' e f g h r)
+set' e f g h (R l a r) = h l a r (set' e f g h l) (set' e f g h r)
 
 {-
  - Construction
@@ -236,13 +257,13 @@ fromDistinctDescList = go <*> power
 
 -- | /O(m log n)/ Subtract a set by another
 difference :: (Ord a) => Set a -> Set a -> Set a
-difference t1 t2 = set empty go go go t1
+difference t1 t2 = set' empty go go go t1
  where
   go _ _ _ _ _ = foldr delete t1 t2
 
 -- | /O(m log n)/ Intersect a set with another
 intersection :: (Ord a) => Set a -> Set a -> Set a
-intersection t1 t2 = set empty go go go t2
+intersection t1 t2 = set' empty go go go t2
  where
   go _ _ _ _ _ =
     fromDistinctAscList $
@@ -253,7 +274,7 @@ intersection t1 t2 = set empty go go go t2
 
 -- | /O(m log n)/ Unite a set with another
 union :: (Ord a) => Set a -> Set a -> Set a
-union t1 t2 = set t2 go go go t1
+union t1 t2 = set' t2 go go go t1
  where
   go _ _ _ _ _ = foldr insert t1 t2
 
@@ -335,19 +356,19 @@ splitMin t = ((,) <*> flip delete' t) <$> lookupMin t
 
 -- | /O(m log n)/ Check whether two sets have no elements in common
 disjoint :: (Ord a) => Set a -> Set a -> Bool
-disjoint t1 t2 = set True go go go t1
+disjoint t1 t2 = set' True go go go t1
  where
   go _ _ _ _ _ = not $ any (`member` t1) t2
 
 -- | /O(n log m)/ Check whether the elements of a set exist in the other
 isSubsetOf :: (Ord a) => Set a -> Set a -> Bool
-isSubsetOf t1 t2 = set (null t1) go go go t2
+isSubsetOf t1 t2 = set' (null t1) go go go t2
  where
   go _ _ _ _ _ = all (`member` t2) t1
 
 -- | /O(log n)/ Fetch the least element greater than or equal to the given one
 lookupGE :: (Ord a) => a -> Set a -> Maybe a
-lookupGE a0 = set Nothing go go go
+lookupGE a0 = set' Nothing go go go
  where
   go _ a _ recl recr = case compare a a0 of
     LT -> recr
@@ -356,7 +377,7 @@ lookupGE a0 = set Nothing go go go
 
 -- | /O(log n)/ Fetch the least element strictly greater than the given one
 lookupGT :: (Ord a) => a -> Set a -> Maybe a
-lookupGT a0 = set Nothing go go go
+lookupGT a0 = set' Nothing go go go
  where
   go _ a _ recl recr = case compare a a0 of
     LT -> recr
@@ -365,7 +386,7 @@ lookupGT a0 = set Nothing go go go
 
 -- | /O(log n)/ Fetch the greatest element less than or equal to the given one
 lookupLE :: (Ord a) => a -> Set a -> Maybe a
-lookupLE a0 = set Nothing go go go
+lookupLE a0 = set' Nothing go go go
  where
   go _ a _ recl recr = case compare a a0 of
     LT -> recr <|> Just a
@@ -374,7 +395,7 @@ lookupLE a0 = set Nothing go go go
 
 -- | /O(log n)/ Fetch the greatest element strictly less than the given one
 lookupLT :: (Ord a) => a -> Set a -> Maybe a
-lookupLT a0 = set Nothing go go go
+lookupLT a0 = set' Nothing go go go
  where
   go _ a _ recl recr = case compare a a0 of
     LT -> recr <|> Just a
@@ -383,23 +404,23 @@ lookupLT a0 = set Nothing go go go
 
 -- | /O(log n)/ Fetch the maximum element, or 'Nothing' if the set is empty
 lookupMax :: Set a -> Maybe a
-lookupMax = set Nothing go go go
+lookupMax = set' Nothing go go go
  where
-  go _ a r _ recr = set (Just a) go' go' go' r
+  go _ a r _ recr = set' (Just a) go' go' go' r
    where
     go' _ _ _ _ _ = recr
 
 -- | /O(log n)/ Fetch the minimum element, or 'Nothing' if the set is empty
 lookupMin :: Set a -> Maybe a
-lookupMin = set Nothing go go go
+lookupMin = set' Nothing go go go
  where
-  go l a _ recl _ = set (Just a) go' go' go' l
+  go l a _ recl _ = set' (Just a) go' go' go' l
    where
     go' _ _ _ _ _ = recl
 
 -- | /O(log n)/ Check whether an element is in a set
 member :: (Ord a) => a -> Set a -> Bool
-member a0 = set False go go go
+member a0 = set' False go go go
  where
   go _ a _ recl recr = case compare a0 a of
     LT -> recl
@@ -408,11 +429,11 @@ member a0 = set False go go go
 
 -- | /O(1)/ Check whether a set is empty
 null :: Set a -> Bool
-null = set True go go go where go _ _ _ _ _ = False
+null = set' True go go go where go _ _ _ _ _ = False
 
 -- | /O(n)/ Get the size of a set
 size :: Set a -> Int
-size = set 0 go go go where go _ _ _ recl recr = 1 + recl + recr
+size = set' 0 go go go where go _ _ _ recl recr = 1 + recl + recr
 
 {-
  - Validation
@@ -423,19 +444,19 @@ valid :: (Ord a) => Set a -> Bool
 valid = liftA2 (&&) balanced ordered
  where
   balanced =
-    set
+    set'
       True
       (\l _ r recl recr -> levels l - levels r == 1 && recl && recr)
       (\l _ r recl recr -> levels l - levels r == 0 && recl && recr)
       (\l _ r recl recr -> levels r - levels l == 1 && recl && recr)
-  levels = set 0 go go go
+  levels = set' 0 go go go
    where
     go _ _ _ recl recr = 1 + max recl recr :: Int
-  ordered = set True go go go
+  ordered = set' True go go go
    where
     go l a r recl recr =
-      set True lt lt lt l
-        && set True gt gt gt r
+      set' True lt lt lt l
+        && set' True gt gt gt r
      where
       lt _ la _ _ _ = la < a && recl && recr
       gt _ ra _ _ _ = ra > a && recl && recr
@@ -466,7 +487,7 @@ power as = until (> length as) (* 2) 2 `div` 2
 -- O(log n) Delete an element from a set without checking for membership
 delete' :: (Ord a) => a -> Set a -> Set a
 delete' a0 =
-  set
+  set'
     (error "Set.delete: L0")
     ( \l a r _ _ ->
         case compare a0 a of
@@ -488,7 +509,7 @@ delete' a0 =
     )
  where
   deleteRl l a r =
-    set
+    set'
       (error "Set.delete: L1")
       ( \ll la lr _ _ ->
           case compare a0 la of
@@ -510,7 +531,7 @@ delete' a0 =
       )
       l
   deleteRr l a =
-    set
+    set'
       (error "Set.delete: L2")
       ( \rl ra rr _ _ ->
           case compare a0 ra of
@@ -531,7 +552,7 @@ delete' a0 =
             GT -> checkRightR l a (deleteRr rl ra rr)
       )
   deleteBl l a r =
-    set
+    set'
       (error "Set.delete: L3")
       ( \ll la lr _ _ ->
           case compare a0 la of
@@ -553,7 +574,7 @@ delete' a0 =
       )
       l
   deleteBr l a =
-    set
+    set'
       (error "Set.delete: L4")
       ( \rl ra rr _ _ ->
           case compare a0 ra of
@@ -574,7 +595,7 @@ delete' a0 =
             GT -> checkRightB l a (deleteRr rl ra rr)
       )
   deleteLl l a r =
-    set
+    set'
       (error "Set.delete: L5")
       ( \ll la lr _ _ ->
           case compare a0 la of
@@ -596,7 +617,7 @@ delete' a0 =
       )
       l
   deleteLr l a =
-    set
+    set'
       (error "Set.delete: L6")
       ( \rl ra rr _ _ ->
           case compare a0 ra of
@@ -617,10 +638,10 @@ delete' a0 =
             GT -> checkRightL l a (deleteRr rl ra rr)
       )
   rebalanceR l a =
-    set
+    set'
       (error "Set.delete: L7")
       ( \rl ra rr _ _ ->
-          set
+          set'
             (error "Set.delete: L8")
             (\rll rla rlr _ _ -> B (B l a rll) rla (R rlr ra rr))
             (\rll rla rlr _ _ -> B (B l a rll) rla (B rlr ra rr))
@@ -630,12 +651,12 @@ delete' a0 =
       (\rl ra rr _ _ -> L (R l a rl) ra rr)
       (\rl ra rr _ _ -> B (B l a rl) ra rr)
   rebalanceL l a r =
-    set
+    set'
       (error "Set.delete: L9")
       (\ll la lr _ _ -> B ll la (B lr a r))
       (\ll la lr _ _ -> R ll la (L lr a r))
       ( \ll la lr _ _ ->
-          set
+          set'
             (error "Set.delete: L10")
             (\lrl lra lrr _ _ -> B (B ll la lrl) lra (R lrr a r))
             (\lrl lra lrr _ _ -> B (B ll la lrl) lra (B lrr a r))
@@ -644,117 +665,117 @@ delete' a0 =
       )
       l
   checkLeftR l a r =
-    set
+    set'
       (error "Set.delete: L11")
       (\_ _ _ _ _ -> R l a r)
       (\_ _ _ _ _ -> rebalanceR l a r)
       (\_ _ _ _ _ -> R l a r)
       l
   checkLeftB l a r =
-    set
+    set'
       (error "Set.delete: L12")
       (\_ _ _ _ _ -> B l a r)
       (\_ _ _ _ _ -> R l a r)
       (\_ _ _ _ _ -> B l a r)
       l
   checkLeftL l a r =
-    set
+    set'
       (error "Set.delete: L13")
       (\_ _ _ _ _ -> L l a r)
       (\_ _ _ _ _ -> B l a r)
       (\_ _ _ _ _ -> L l a r)
       l
   checkRightR l a r =
-    set
+    set'
       (error "Set.delete: L14")
       (\_ _ _ _ _ -> R l a r)
       (\_ _ _ _ _ -> B l a r)
       (\_ _ _ _ _ -> R l a r)
       r
   checkRightB l a r =
-    set
+    set'
       (error "Set.delete: L15")
       (\_ _ _ _ _ -> B l a r)
       (\_ _ _ _ _ -> L l a r)
       (\_ _ _ _ _ -> B l a r)
       r
   checkRightL l a r =
-    set
+    set'
       (error "Set.delete: L16")
       (\_ _ _ _ _ -> L l a r)
       (\_ _ _ _ _ -> rebalanceL l a r)
       (\_ _ _ _ _ -> L l a r)
       r
   substituteR l =
-    set
+    set'
       (error "Set.delete: L17")
       (\rl ra rr _ _ -> uncurry (checkRightR l) $ popLeftL rl ra rr)
       (\rl ra rr _ _ -> uncurry (checkRightR' l) $ popLeftB rl ra rr)
       (\rl ra rr _ _ -> uncurry (checkRightR l) $ popLeftR rl ra rr)
   substituteBr l =
-    set
+    set'
       E
       (\rl ra rr _ _ -> uncurry (checkRightB l) $ popLeftL rl ra rr)
       (\rl ra rr _ _ -> uncurry (checkRightB' l) $ popLeftB rl ra rr)
       (\rl ra rr _ _ -> uncurry (checkRightB l) $ popLeftR rl ra rr)
   substituteBl l r =
-    set
+    set'
       E
       (\ll la lr _ _ -> (\(l', a) -> checkLeftB l' a r) $ popRightL ll la lr)
       (\ll la lr _ _ -> (\(l', a) -> checkLeftB' l' a r) $ popRightB ll la lr)
       (\ll la lr _ _ -> (\(l', a) -> checkLeftB l' a r) $ popRightR ll la lr)
       l
   substituteL l r =
-    set
+    set'
       (error "Set.delete: L18")
       (\ll la lr _ _ -> (\(l', a) -> checkLeftL l' a r) $ popRightL ll la lr)
       (\ll la lr _ _ -> (\(l', a) -> checkLeftL' l' a r) $ popRightB ll la lr)
       (\ll la lr _ _ -> (\(l', a) -> checkLeftL l' a r) $ popRightR ll la lr)
       l
   checkLeftR' l a r =
-    set
+    set'
       (rebalanceR l a r)
       (\_ _ _ _ _ -> R l a r)
       (\_ _ _ _ _ -> R l a r)
       (\_ _ _ _ _ -> R l a r)
       l
   checkLeftB' l a r =
-    set
+    set'
       (R l a r)
       (\_ _ _ _ _ -> B l a r)
       (\_ _ _ _ _ -> B l a r)
       (\_ _ _ _ _ -> B l a r)
       l
   checkLeftL' l a r =
-    set
+    set'
       (B l a r)
       (\_ _ _ _ _ -> L l a r)
       (\_ _ _ _ _ -> L l a r)
       (\_ _ _ _ _ -> L l a r)
       l
   checkRightR' l a r =
-    set
+    set'
       (B l a r)
       (\_ _ _ _ _ -> R l a r)
       (\_ _ _ _ _ -> R l a r)
       (\_ _ _ _ _ -> R l a r)
       r
   checkRightB' l a r =
-    set
+    set'
       (L l a r)
       (\_ _ _ _ _ -> B l a r)
       (\_ _ _ _ _ -> B l a r)
       (\_ _ _ _ _ -> B l a r)
       r
   checkRightL' l a r =
-    set
+    set'
       (rebalanceL l a r)
       (\_ _ _ _ _ -> L l a r)
       (\_ _ _ _ _ -> L l a r)
       (\_ _ _ _ _ -> L l a r)
       r
   popLeftR l a r =
-    set
+    set'
       (a, r)
       ( \ll la lr _ _ ->
           (\(a', l') -> (a', checkLeftR l' a r)) $
@@ -767,14 +788,14 @@ delete' a0 =
       )
       l
   popLeftB l a r =
-    set
+    set'
       (a, E)
       (\ll la lr _ _ -> popLeftBL ll la lr a r)
       (\ll la lr _ _ -> popLeftBB ll la lr a r)
       (\ll la lr _ _ -> popLeftBR ll la lr a r)
       l
   popLeftL l a r =
-    set
+    set'
       (error "Set.delete: L19")
       ( \ll la lr _ _ ->
           (\(a', l') -> (a', checkLeftL l' a r)) $
@@ -787,7 +808,7 @@ delete' a0 =
       )
       l
   popLeftRB ll la lr a r =
-    set
+    set'
       (la, rebalanceR E a r)
       ( \lll lla llr _ _ ->
           (\(a', l) -> (a', R l a r)) $
@@ -803,7 +824,7 @@ delete' a0 =
       )
       ll
   popLeftBB ll la lr a r =
-    set
+    set'
       (la, R E a r)
       ( \lll lla llr _ _ ->
           (\(a', l) -> (a', B l a r)) $
@@ -819,7 +840,7 @@ delete' a0 =
       )
       ll
   popLeftLB ll la lr a r =
-    set
+    set'
       (la, B E a E)
       ( \lll lla llr _ _ ->
           (\(a', l) -> (a', L l a r)) $
@@ -841,37 +862,37 @@ delete' a0 =
     (\(a', l) -> (a', checkLeftB l a r)) $
       popLeftL ll la lr
   popRightR l a =
-    set
+    set'
       (error "Set.delete: L20")
       (\rl ra rr _ _ -> first (checkRightR l a) $ popRightL rl ra rr)
       (\rl ra rr _ _ -> popRightRB l a rl ra rr)
       (\rl ra rr _ _ -> first (checkRightR l a) $ popRightR rl ra rr)
   popRightB l a =
-    set
+    set'
       (E, a)
       (\rl ra rr _ _ -> popRightBL l a rl ra rr)
       (\rl ra rr _ _ -> popRightBB l a rl ra rr)
       (\rl ra rr _ _ -> popRightBR l a rl ra rr)
   popRightL l a =
-    set
+    set'
       (l, a)
       (\rl ra rr _ _ -> first (checkRightL l a) $ popRightL rl ra rr)
       (\rl ra rr _ _ -> popRightLB l a rl ra rr)
       (\rl ra rr _ _ -> first (checkRightL l a) $ popRightR rl ra rr)
   popRightRB l a rl ra =
-    set
+    set'
       (B E a E, ra)
       (\rrl rra rrr _ _ -> first (R l a) $ popRightBL rl ra rrl rra rrr)
       (\rrl rra rrr _ _ -> first (R l a) $ popRightBB rl ra rrl rra rrr)
       (\rrl rra rrr _ _ -> first (R l a) $ popRightBR rl ra rrl rra rrr)
   popRightBB l a rl ra =
-    set
+    set'
       (L l a E, ra)
       (\rrl rra rrr _ _ -> first (B l a) $ popRightBL rl ra rrl rra rrr)
       (\rrl rra rrr _ _ -> first (B l a) $ popRightBB rl ra rrl rra rrr)
       (\rrl rra rrr _ _ -> first (B l a) $ popRightBR rl ra rrl rra rrr)
   popRightLB l a rl ra =
-    set
+    set'
       (rebalanceL l a E, ra)
       (\rrl rra rrr _ _ -> first (L l a) $ popRightBL rl ra rrl rra rrr)
       (\rrl rra rrr _ _ -> first (L l a) $ popRightBB rl ra rrl rra rrr)
@@ -882,7 +903,7 @@ delete' a0 =
 -- O(log n) Insert an element into a set without checking for membership
 insert' :: (Ord a) => a -> Set a -> Set a
 insert' a0 =
-  set
+  set'
     (B E a0 E)
     (\l a r _ _ -> insertL l a r)
     (\l a r _ _ -> insertB l a r)
@@ -904,12 +925,12 @@ insert' a0 =
       EQ -> L l a0 r
       GT -> insertLr l a r
   insertRl l a r =
-    set
+    set'
       (B (B E a0 E) a r)
       (\ll la lr _ _ -> R (insertL ll la lr) a r)
       ( \ll la lr _ _ ->
           let l' = insertB ll la lr
-           in set
+           in set'
                 (error "Set.insert: L0")
                 (\_ _ _ _ _ -> B l' a r)
                 (\_ _ _ _ _ -> R l' a r)
@@ -919,12 +940,12 @@ insert' a0 =
       (\ll la lr _ _ -> R (insertR ll la lr) a r)
       l
   insertBl l a r =
-    set
+    set'
       (L (B E a0 E) a r)
       (\ll la lr _ _ -> B (insertL ll la lr) a r)
       ( \ll la lr _ _ ->
           let l' = insertB ll la lr
-           in set
+           in set'
                 (error "Set.insert: L1")
                 (\_ _ _ _ _ -> L l' a r)
                 (\_ _ _ _ _ -> B l' a r)
@@ -934,12 +955,12 @@ insert' a0 =
       (\ll la lr _ _ -> B (insertR ll la lr) a r)
       l
   insertBr l a =
-    set
+    set'
       (R l a (B E a0 E))
       (\rl ra rr _ _ -> B l a (insertL rl ra rr))
       ( \rl ra rr _ _ ->
           let r = insertB rl ra rr
-           in set
+           in set'
                 (error "Set.insert: L2")
                 (\_ _ _ _ _ -> R l a r)
                 (\_ _ _ _ _ -> B l a r)
@@ -948,12 +969,12 @@ insert' a0 =
       )
       (\rl ra rr _ _ -> B l a (insertR rl ra rr))
   insertLr l a =
-    set
+    set'
       (B l a (B E a0 E))
       (\rl ra rr _ _ -> L l a (insertL rl ra rr))
       ( \rl ra rr _ _ ->
           let r = insertB rl ra rr
-           in set
+           in set'
                 (error "Set.insert: L3")
                 (\_ _ _ _ _ -> B l a r)
                 (\_ _ _ _ _ -> L l a r)
@@ -962,7 +983,7 @@ insert' a0 =
       )
       (\rl ra rr _ _ -> L l a (insertR rl ra rr))
   insertRr l a =
-    set
+    set'
       (error "Set.insert: L4")
       (\rl ra rr _ _ -> R l a (insertL rl ra rr))
       ( \rl ra rr _ _ ->
@@ -973,7 +994,7 @@ insert' a0 =
       )
       (\rl ra rr _ _ -> R l a (insertR rl ra rr))
   insertLl l a r =
-    set
+    set'
       (error "Set.insert: L5")
       (\ll la lr _ _ -> L (insertL ll la lr) a r)
       ( \ll la lr _ _ ->
@@ -985,12 +1006,12 @@ insert' a0 =
       (\ll la lr _ _ -> L (insertR ll la lr) a r)
       l
   insertRrr l a rl ra =
-    set
+    set'
       (B (B l a rl) ra (B E a0 E))
       (\rrl rra rrr _ _ -> R l a (B rl ra (insertL rrl rra rrr)))
       ( \rrl rra rrr _ _ ->
           let rr = insertB rrl rra rrr
-           in set
+           in set'
                 (error "Set.insert: L6")
                 (\_ _ _ _ _ -> B (B l a rl) ra rr)
                 (\_ _ _ _ _ -> R l a (B rl ra rr))
@@ -999,12 +1020,12 @@ insert' a0 =
       )
       (\rrl rra rrr _ _ -> R l a (B rl ra (insertR rrl rra rrr)))
   insertLll ll la lr a r =
-    set
+    set'
       (B (B E a0 E) la (B lr a r))
       (\lll lla llr _ _ -> L (B (insertL lll lla llr) la lr) a r)
       ( \lll lla llr _ _ ->
           let ll' = insertB lll lla llr
-           in set
+           in set'
                 (error "Set.insert: L7")
                 (\_ _ _ _ _ -> B ll' la (B lr a r))
                 (\_ _ _ _ _ -> L (B ll' la lr) a r)
@@ -1014,12 +1035,12 @@ insert' a0 =
       (\lll lla llr _ _ -> L (B (insertR lll lla llr) la lr) a r)
       ll
   insertRrl l a rl ra rr =
-    set
+    set'
       (B (B l a E) a0 (B E ra rr))
       (\rll rla rlr _ _ -> R l a (B (insertL rll rla rlr) ra rr))
       ( \rll rla rlr _ _ ->
           let rl' = insertB rll rla rlr
-           in set
+           in set'
                 (error "Set.insert: L8")
                 (\rll' rla' rlr' _ _ -> B (B l a rll') rla' (R rlr' ra rr))
                 (\_ _ _ _ _ -> R l a (B rl' ra rr))
@@ -1029,12 +1050,12 @@ insert' a0 =
       (\rll rla rlr _ _ -> R l a (B (insertR rll rla rlr) ra rr))
       rl
   insertLlr ll la lr a r =
-    set
+    set'
       (B (B ll la E) a0 (B E a r))
       (\lrl lra lrr _ _ -> L (B ll la (insertL lrl lra lrr)) a r)
       ( \lrl lra lrr _ _ ->
           let lr' = insertB lrl lra lrr
-           in set
+           in set'
                 (error "Set.insert: L9")
                 (\lrl' lra' lrr' _ _ -> B (B ll la lrl') lra' (R lrr' a r))
                 (\_ _ _ _ _ -> L (B ll la lr') a r)
